#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a new blackwhite image framed (using G'MIC filters)

# (c) Dinasset 2023
#
#   History:
#
#   v 1.0.0: 2023-08   : Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
from gimpfu import *

# define here some constant value, when required
maxwh=5000

############################################################################
# Service routines
#---------------------------------------------------------------------------
# from Original drawable create the Output image
#---------------------------------------------------------------------------
def createOutimage(inImage, inLayer, widthheight, prepstep):
     
    owidth = inImage.width
    oheight = inImage.height
    otype = inImage.base_type
    oname = inLayer.name
    
   # define/calculate the scale factor for the (new) output image -------------------------
    realWidthHeight = owidth+oheight      # real size of the input image
    if widthheight == 0 :   # means the user wants to have output size same as input size
        if realWidthHeight<=maxwh:
            widthheight = realWidthHeight
        else:
            widthheight = maxwh  # max
            gimp.message ("forced max width+height")
    optiWidthHeight = widthheight       # size to work on to get best results
    scale = (optiWidthHeight*1.0)/(realWidthHeight*1.0)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    
   # create a new image of the decided size ------------------------------------------------
   # --- copy the source layer
   # --- scale
    newImage = pdb.gimp_image_new (rwidth, rheight, otype)                    
    newBG = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    newBG.mode = LAYER_MODE_NORMAL
    newBG.name = "NewImage: Background Layer from "+oname
    newBG.opacity = 100.0 
    newImage.add_layer(newBG, 0)
    pdb.gimp_layer_scale(newBG, rwidth, rheight, False)
    pdb.gimp_layer_add_alpha (newBG)    # RGBA
    
   # keep a copy of the original background below the one which will be used ----------------
   # --- and set its visibility off (it should not enter in the process)
    Original = newBG.copy()
    newImage.add_layer(Original, 1)     # below
    pdb.gimp_item_set_visible (Original, False)
    Original.name = oname+" (original)"
    
    # create base for effect
    pdb.gimp_image_set_active_layer (newImage,  newBG)

    # optional user choice for preprocessing
    if prepstep>0:  #user did a selection
        #               #"DCP Dehaze (good for light image",
        #               #"Dynamic Range Increase",
        #               #"Equalize Local Histogram",
        #               #"Local Contrast Enhancement",
        #               #"Local Normalization",
        #               #"Local Processing (Equalize)",
        #               #"Local Processing (Normalize)",
        if prepstep==1: # jeje_dehaze 5,1,0.2,1,0,0,0,0,0
            parms='5,1,0.2,1,0,0,0,0,0'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -jeje_dehaze "+str(parms)+" ")
        elif prepstep==2: # fx_tk_dri 0.5,0.5,0.5,2.5,0.5,1,1,0
            parms='0.5,0.5,0.5,2.5,0.5,1,1,0'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_tk_dri "+str(parms)+" ")
        elif prepstep==3: # fx_equalize_local_histograms 75,2,4,100,8,1,16,0,50,50
            parms='75,2,4,100,8,1,16,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_equalize_local_histograms "+str(parms)+" ")
        elif prepstep==4: # fx_LCE 80,0.5,1,1,0,0
            parms='80,0.5,1,1,0,0'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_LCE "+str(parms)+" ")
        elif prepstep==5: # fx_normalize_local 2,6,5,20,1,11,0,50,50
            parms='2,6,5,20,1,11,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_normalize_local "+str(parms)+" ")
        elif prepstep==6: # fx_local_processing 1,75,10,50,20,0,7,0,50,50
            parms='1,75,10,50,20,0,7,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_local_processing "+str(parms)+" ")
        elif prepstep==7: # fx_local_processing 0,75,10,50,20,0,7,0,50,50
            parms='0,75,10,50,20,0,7,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_local_processing "+str(parms)+" ")
    newBG = pdb.gimp_image_get_active_layer (newImage)
            
    # adjust mean value of the base
    # "normal" mean value should be betweeb 117.5 and 137.5
    (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (newBG, HISTOGRAM_LUMINANCE, 0.0, 1.0)
    #gimp.message ("mean="+str(mean))
    if mean<117.5:
        #gimp.message ("mean before="+str(mean))
        loop=0
        while (mean<117.5) and (loop<100):  
            pdb.gimp_drawable_levels (newBG, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.1, 0.0, 1.0, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (newBG,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
            loop+=1
        #gimp.message ("mean after="+str(mean))
        newBG.name = newBG.name+" mean adj"
    elif mean>137.5:
        #gimp.message ("mean before="+str(mean))
        loop=0
        while (mean>137.5) and (loop<100):  
            pdb.gimp_drawable_levels (newBG, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.9, 0.0, 1.0, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (newBG,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
            loop+=1
        #gimp.message ("mean after="+str(mean))
        newBG.name = newBG.name+" mean adj"
    newBG.opacity = 100.0
    
    return(newImage)
#--------------------------------------------------------------------------- 
    
############################################################################
# Service routine (by MareroQ) for Frame colour
#---------------------------------------------------------------------------
def gmic_rgba_to_rgb(rgba_color):

	r = rgba_color[0]
	g = rgba_color[1]
	b = rgba_color[2]

	# write rgb in correct syntax
	rgb_color = "{0},{1},{2}".format(r,g,b)
	return rgb_color
	     
############################################################################
# Main
#---------------------------------------------------------------------------

def Black_WhiteFramed (inImage, inLayer, widthheight,
                        frame, size_fr, color_frame,
                        style,
                        regaincolours, prepstep,
                        doFlatten):
      
    # here the logic of the filter
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    # auto mode and opacity
    blackwhitemode=0			#fixed auto
    blackwhiteopacity=0		    #fixed auto
    	
    newImage = createOutimage (inImage, inLayer, widthheight, prepstep)
    newBGo = newImage.layers[0]
    newBGo.name = "new BG original"
    
    # create base for applying chosen style
    baseforeffect = newBGo.copy()
    newImage.add_layer (baseforeffect, 0)
    baseforeffect.name = "base for effect"
    pdb.gimp_drawable_desaturate (baseforeffect, DESATURATE_LUMA)
    
    imgW = newImage.width
    imgH = newImage.height
    imgsize = imgW+imgH
    
    # add local contrast as a preliminary step
    # afre_localcontrast 2,100
    pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,0,
        "-v - -afre_localcontrast 2,100 ") 
         
    # set defaults mode/opacity
    autolayermode = LAYER_MODE_SOFTLIGHT  # default mode
    autolayeropacity = 75.0  # default opacity
    
    # COMMON START
    StyleLayer = baseforeffect.copy()
    newImage.add_layer (StyleLayer, 0)
    
    # apply chosen GMIC filter: Box_Fitting effect
    if style==0:    #B&W Stencil
        # Apply stencilbw
        #[G'MIC] fx_stencilbw 4,4,0,0,0,50,50
    
        parms='4,4,0,0,0,50,50'

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_stencilbw "+str(parms)+" ")
        
        StyleLayer.name = "Black_White: fx_stencilbw "+str(parms)
         
        if blackwhitemode==0: autolayermode = LAYER_MODE_SOFTLIGHT
        if blackwhiteopacity<1: autolayeropacity = 50.0
                                         
        #pdb.gimp_display_new (newImage)
        #return
         
    elif style==1:    #BlackandWhite
        # Apply blackandwhite
        #[G'MIC] fx_blackandwhite 0.299,0,0.587,0,0.114,0,0,0,0,0,0,4,4,4,1.91,0,0,60,16,4,0,0,0,50,50
        
        parms='0.299,0,0.587,0,0.114,0,0,0,0,0,0,4,4,4,1.91,0,0,60,16,4,0,0,0,50,50'

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_blackandwhite "+str(parms)+" ")
        
        StyleLayer.name = "Black_White: fx_blackandwhite "+str(parms)
        
        if blackwhitemode==0: autolayermode = LAYER_MODE_GRAIN_MERGE
        if blackwhiteopacity<1: autolayeropacity = 50.0
              
        #pdb.gimp_display_new (newImage)
        #return
        
    elif style==2:    #Charcoal
        # apply charcoal
        #[G'MIC] fx_charcoal 800,0,128,0,1,1,31,33,255,255,255,0,0,0,0,0,50,50
         
        parms='800,0,128,0,1,1,31,33,255,255,255,0,0,0,0,0,50,50'

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_charcoal "+str(parms)+" ")
        
        StyleLayer.name = "Black_White: fx_charcoal "+str(parms)
         
        if blackwhitemode==0: autolayermode = LAYER_MODE_HSV_VALUE
        if blackwhiteopacity<1: autolayeropacity = 35.0
              
        #pdb.gimp_display_new (newImage)
        #return
        
    elif style==3:    #Dithering
        # apply ditheredbw
        #[G'MIC] fx_ditheredbw 0,0,0,0,0,0,0,50,50
    
        parms='0,0,0,0,0,0,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_ditheredbw "+str(parms)+" ") 
        
        StyleLayer.name = "Black_White: fx_ditheredbw "+str(parms)
         
        if blackwhitemode==0: autolayermode = LAYER_MODE_VIVID_LIGHT
        if blackwhiteopacity<1: autolayeropacity = 50.0
           
        #pdb.gimp_display_new (newImage)
        #return
        
    elif style==4:    #Emboss
        # apply emboss
        #[G'MIC] gcd_emboss 160,0
    
        parms='160,0'

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -gcd_emboss "+str(parms)+" ")     
        
        StyleLayer.name = "Black_White: gcd_emboss "+str(parms)
         
        if blackwhitemode==0: autolayermode = LAYER_MODE_LINEAR_LIGHT
        if blackwhiteopacity<1: autolayeropacity = 60.0
           
        #pdb.gimp_display_new (newImage)
        #return
        
    elif style==5:    #Engrave
        # apply engrave
        #[G'MIC] fx_engrave 0.5,50,0,8,40,0,0,0,10,1,0,0,0,1,0,50,50
    
        parms='0.5,50,0,40,40,0,0,0,10,1,0,0,0,1,0,50,50'

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - fx_engrave "+str(parms)+" ")
        
        StyleLayer.name = "Black_White: fx_engrave "+str(parms)
         
        if blackwhitemode==0: autolayermode = LAYER_MODE_VIVID_LIGHT
        if blackwhiteopacity<1: autolayeropacity = 95.0
           
        #pdb.gimp_display_new (newImage)
        #return
        
    elif style==6:    #FreakyBW
        # apply freaky_bw
        #[G'MIC] fx_freaky_bw 50,100,0,20,0,0,50,50
        
        parms='50,100,0,20,0,0,50,50'

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_freaky_bw "+str(parms)+" ")     
        
        StyleLayer.name = "Black_White: fx_freaky_bwj "+str(parms)
         
        if blackwhitemode==0: autolayermode = LAYER_MODE_SCREEN
        if blackwhiteopacity<1: autolayeropacity = 65.0
           
        #pdb.gimp_display_new (newImage)
        #return
        
    elif style==7:    #InkWash
        # apply ink_wash
        #[G'MIC] fx_ink_wash 0.14,23,0,0.5,0.54,2.25,0,2,6,5,20
        
        parms='0.14,23,0,0.5,0.54,2.25,0,2,6,5,20'

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_ink_wash "+str(parms)+" ")     
        
        StyleLayer.name = "Black_White: fx_ink_wash "+str(parms)
         
        if blackwhitemode==0: autolayermode = LAYER_MODE_LINEAR_LIGHT
        if blackwhiteopacity<1: autolayeropacity = 50.0
           
        #pdb.gimp_display_new (newImage)
        #return
        
    elif style==8:    #LayerEtch
        # apply layeretch
        #[G'MIC] fx_gcd_layeretch 11,4,12,0.12,100,8.5,5,0,0,3,1,1,0
        
        parms='11,4,12,0.12,100,8.5,5,0,0,3,1,1,0'
        
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_gcd_layeretch "+str(parms)+" ")     
        
        StyleLayer.name = "Black_White: fx_gcd_layeretch  "+str(parms)
         
        if blackwhitemode==0: autolayermode = LAYER_MODE_SCREEN
        if blackwhiteopacity<1: autolayeropacity = 95.0
           
        #pdb.gimp_display_new (newImage)
        #return
      
    elif style==9:    #PencilBW
        # apply pencilbw
        #[G'MIC] fx_pencilbw 3,15,0,0,0,50,50

        parms='3,15,0,0,0,50,50'

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_pencilbw "+str(parms)+" ")     
        
        StyleLayer.name = "Black_White: fx_pencilbw "+str(parms)
         
        if blackwhitemode==0: autolayermode = LAYER_MODE_ADDITION
        if blackwhiteopacity<1: autolayeropacity = 75.0
           
        #pdb.gimp_display_new (newImage)
        #return
      
    elif style==10:    #PencilPortraitBW
        # apply pencil_portraitbw
        #[G'MIC] fx_pencil_portraitbw 30,120,0,0.5,48,8,8,0,50,50

        parms='30,120,0,0.5,48,8,8,0,50,50'

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_pencil_portraitbw "+str(parms)+" ")     
        
        StyleLayer.name = "Black_White: fx_pencil_portraitbw "+str(parms)
         
        if blackwhitemode==0: autolayermode = LAYER_MODE_LUMINANCE
        if blackwhiteopacity<1: autolayeropacity = 95.0
           
        #pdb.gimp_display_new (newImage)
        #return
      
    elif style==11:    #Etch
        # apply etch
        #[G'MIC] fx_gcd_etch 125,153,171,185,0.1,60,50,50,10,15,12,20,0,1,0.3,1,0,0
        
        parms='125,153,171,185,0.1,60,50,50,10,15,12,20,0,1,0.3,1,0,0'

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_gcd_etch "+str(parms)+" ")     
        
        StyleLayer.name = "Black_White: fx_gcd_etch "+str(parms)
         
        if blackwhitemode==0: autolayermode = LAYER_MODE_GRAIN_EXTRACT  #
        if blackwhiteopacity<1: autolayeropacity = 50.0 #
           
    #pdb.gimp_display_new (newImage)
    #return
      
    # COMMON ******************************************************
    
    ArtisticLayer = pdb.gimp_image_get_active_layer(newImage)
    if blackwhitemode==0: ArtisticLayer.mode = autolayermode
    if blackwhiteopacity<1: ArtisticLayer.opacity = autolayeropacity
    
    # for color restore (user choice)
    forcolors = newBGo.copy()
    newImage.add_layer (forcolors, 0)
    forcolors.mode = LAYER_MODE_GRAIN_MERGE # OVERLAY # HARDLIGHT   # LCH_COLOR
    forcolors.opacity = regaincolours*1.0
    forcolors.name = "Original to re-apply colors"
    pdb.gimp_drawable_hue_saturation (forcolors, HUE_RANGE_ALL,
                                      0.0,0.0,100.0,0.0)
    #pdb.gimp_display_new (newImage)
    #return
    final_layer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                   "Pre_Outcome")
    newImage.add_layer (final_layer, 0)
    final_layer.name = "Preliminary Outcome (unframed)"
    (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (final_layer,
                                   HISTOGRAM_LUMINANCE, 0.0, 1.0)
    #pdb.gimp_message ("mean Preliminary Outcome="+str(mean))
    if mean < 75.5:
        final_layer.mode = LAYER_MODE_DODGE
    elif mean < 107.5:
        final_layer.mode = LAYER_MODE_ADDITION
    elif mean > 179.5:
        final_layer.mode = LAYER_MODE_MULTIPLY
    elif mean > 147.5:
        final_layer.mode = LAYER_MODE_OVERLAY
    else:    
        final_layer.mode = LAYER_MODE_NORMAL
    #pdb.gimp_display_new (newImage)
    #return
    
    Framed = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                   "Outcome")
    newImage.add_layer (Framed, 0)
    # apply GMIC Frame Blur  
    # fx_frame_blur 10,10,0,5,1,0,128,128,128,0,0,255,255,255,2,2,1,0,0.5,0.5,0
    hex = gmic_rgba_to_rgb(color_frame)
    SH = int(size_fr)
    SV = int(size_fr)
    Round=["0.0","1.0","0.2","0.15","0.1"]	
    Rnd=str(Round[frame])
    XSh = min(10,int(size_fr/6))
    YSh = min(10,int(size_fr/6))
    OuS = min(10,int(size_fr/3))	
    parms=str(SH)+','+str(SV)+',0,5,'+str(Rnd)+',0,128,128,128,0,'
    parms=parms+str(OuS)+','+str(hex)+','+str(XSh)+','+str(YSh)+',1,0,0.5,0.5,0 '	
    pdb.plug_in_gmic_qt(newImage, final_layer, 1,0,
        "-v - -fx_frame_blur "+str(parms)+" ")
    Framed = pdb.gimp_image_get_active_layer(newImage)
    Framed.mode = LAYER_MODE_NORMAL
 
    if doFlatten==True:
        pdb.gimp_image_remove_layer (newImage, final_layer)
        pdb.gimp_image_remove_layer (newImage, forcolors)
        pdb.gimp_image_remove_layer (newImage, ArtisticLayer)
        pdb.gimp_image_remove_layer (newImage, baseforeffect)
        pdb.gimp_image_remove_layer (newImage, newBGo)        
         
    #pdb.gimp_display_new (newImage)
    #return
     
    stylename = ["BWStencil","BlackWhite","Charcoal", 
                 "Dithering","Emboss","Engrave",
                 "FreakyBW","InkWash","LayerEtch",
                 "Pencil","PencilPortraitBW","Etch"
                 ]     
    Framed.name = st+oname+" with a framed B&W effect, style "
    Framed.name = Framed.name+str(stylename[style])
    # optional user choice for preprocessing
    prepsname = ["",    
                "DCP Dehaze (good for light image",
                "Dynamic Range Increase",
                "Equalize Local Histogram",
                "Local Contrast Enhancement",
                "Local Normalization",
                "Local Processing (Equalize)",
                "Local Processing (Normalize)",
                ]
    
    if prepstep>0:
        Framed.name = Framed.name+" (prep.step choosen="+str(prepsname[prepstep])+"."
                		
    frameDX = imgW*SH/200
    frameDY = imgH*SV/200
    delta = max(frameDX, frameDY)
    bsize = max(imgW, imgH)
    corner=[0.0,100.0,40.0,30.0,20.0]	
    angle=(float(corner[frame]))
    #pdb.gimp_message ("angle="+str(angle))
     	
    if doFlatten==False:
        # create a frame only layer
        # note that the "clear" is approximate, because it seems that
        #      GMIC and GIMP calculate differently the curves
        #      in the vertices of a round rectangle
        FrameOnly = Framed.copy()
        newImage.add_layer (FrameOnly, 1)
        FrameOnly.name = "Frame Only"
        pdb.gimp_layer_add_alpha (FrameOnly)
        if frame==0:
            pdb.gimp_image_select_rectangle (newImage, CHANNEL_OP_REPLACE,
            delta, delta, imgW, imgH)
        elif frame==1:
            pdb.gimp_image_select_ellipse (newImage, CHANNEL_OP_REPLACE,
            delta, delta, imgW, imgH) 	
        elif frame>1:	
            pdb.gimp_image_select_round_rectangle (newImage, CHANNEL_OP_REPLACE,
            delta, delta, imgW, imgH, angle*imgW/100, angle*imgH/100)  	
        pdb.gimp_drawable_edit_clear (FrameOnly)
        pdb.gimp_selection_none (newImage)	
        pdb.gimp_item_set_visible (Framed, False)    
         
    #pdb.gimp_display_new (newImage)
    #return
     
    # align all layers to Framed internal image
    # by MrQ solution	
    for layer in newImage.layers:		
    	pdb.gimp_layer_set_offsets(layer,
    	   (pdb.gimp_image_width(newImage)-pdb.gimp_drawable_width(layer))/2,
    	   (pdb.gimp_image_height(newImage)-pdb.gimp_drawable_height(layer))/2)
    	
    pdb.gimp_display_new (newImage)
             
### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/SimpleFilters/"
copyrightYear="2023"
desc="Black_White Framed"

register(
    "Black_WhiteFramed",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    "Black_White Framed, version 1.1.0 (initial)",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_ADJUSTMENT, "newsize", "Width+Height of New Image"+
         "\n (0=same as input)", 0, (0, 5000, 100)),
        (PF_OPTION, "frame", "Choose frame format", 0,
                     ["square","round","0.2 rounded square",
                      "0.15 rounded square","0.1 rounded square"
                      ]),
        (PF_SPINNER, "size_fr",    "Frame thickness", 20, (0, 50, 2)),					  
        (PF_COLOR,   "color_frame", "Color frame", (200,200,200)),					  
        (PF_OPTION, "effect", "Choose GMIC BlackWhite effect ", 0,
                     ["BWStencil","BlackWhite","Charcoal",
                      "Dithering","Emboss","Engrave",
                      "FreakyBW","InkWash","LayerEtch",
                      "Pencil","PencilPortraitBW","Etch"
                      ]),
        (PF_SPINNER, "getcolours", "Regain colours?", 35, (0, 100, 1)),
        (PF_OPTION, "preproc", "Choose preprocessing step ", 0,
                     ["*none",
                      "DCP Dehaze (for light images)",
                      "Dynamic Range Increase",
                      "Equalize Local Histogram",
                      "Local Contrast Enhancement",
                      "Local Normalization",
                      "Local Processing (Equalize)",
                      "Local Processing (Normalize)",
                      ]),        
        (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [],
    Black_WhiteFramed,
    menu=menu
)

main()
